/* 
   CAM - Cpu's Audio Mixer for Linux 
 
   AUTHOR: Jan 'TWP' VANDENBERGHE (jvdbergh@wins.uia.ac.be) 
*/ 
 
/* 
     This program is free software; you can redistribute it and/or 
     modify it under the terms of the GNU General Public License 
     as published by the Free Software Foundation; either version 2 
     of the License, or (at your option) any later version. 
 
     This program is distributed in the hope that it will be useful, 
     but WITHOUT ANY WARRANTY; without even the implied warranty of 
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
     GNU General Public License for more details. 
 
     You should have received a copy of the GNU General Public License 
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
*/ 
 
#define VERSION_HIGH 0 
#define VERSION_LOW 6
 
#define COLOR_ACT_HANDLE 1 
#define COLOR_NACT_HANDLE 2 
#define COLOR_ACT_SLIDE 3 
#define COLOR_NACT_SLIDE 4 
#define COLOR_STATUS 6 
#define COLOR_MSG 7 
#define COLOR_REC_ON 8 
#define COLOR_REC_OFF 9 
#define COLOR_TITLE 10 
#define COLOR_HELP 11 
#define COLOR_VOL1 12 
#define COLOR_VOL2 13 
#define COLOR_VOL3 14 
 
#define ACT_HANDLE (COLOR_PAIR( COLOR_ACT_HANDLE) | A_BOLD | A_ALTCHARSET) 
#define NACT_HANDLE (COLOR_PAIR( COLOR_NACT_HANDLE) | A_DIM | A_ALTCHARSET) 
 
void status(char *status_line); 
void message(char * message_line); 
 
void stop(void); 
int getbits(int x, int p, int n); 
 
void Draw_handle(int level,int act, int left); 
void Draw_slide(void); 
void put_screen(void); 
 
void help(void); 
void comm_help(void); 

void make_colors(); 

